pub struct GErrSource {
pub id: Option<Box<dyn IdSource>>,
pub id_json: Option<Value>,
pub code: Option<Cow<'static, str>>,
pub message: Cow<'static, str>,
pub sources: Option<Vec<Source>>,
pub tags: Option<Vec<Cow<'static, str>>>,
pub data: Option<Box<dyn DataSource>>,
pub help: Option<Cow<'static, str>>,
pub data_json: Option<Value>,
pub location: Option<ErrorLocation>,
}Expand description
GErrSource - GErr as source of error.
Fields
id: Option<Box<dyn IdSource>>Error id, must implement Debug and Display.
id_json: Option<Value>Error id as json value, support for numeric and string only.
Passing other than numeric and string, will be ignored at serde.
code: Option<Cow<'static, str>>Error code.
message: Cow<'static, str>Error message.
sources: Option<Vec<Source>>Error sources.
Error tags.
data: Option<Box<dyn DataSource>>Error data.
help: Option<Cow<'static, str>>Error help.
data_json: Option<Value>Error data as json values. Supports all json values.
location: Option<ErrorLocation>Error location.